//
// agent2.cc      : Test agent #2 Main File
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: agent2.cc,v 1.15 2002/02/22 01:17:43 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "agent.hh"
#include <unistd.h>

MyReceive *mr;

NRSimpleAttributeFactory<char *> TargetAttr(NRAttribute::TARGET_KEY, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<char *> AppStringAttr(APP_KEY2, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<int> AppCounterAttr(APP_KEY3, NRAttribute::INT32_TYPE);
NRSimpleAttributeFactory<int> AppDummyAttr(APP_KEY1, NRAttribute::INT32_TYPE);

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  NRSimpleAttribute<int> *nrclass = NULL;

  nrclass = NRClassAttr.find(data);

  if (!nrclass){
    diffPrint(DEBUG_ALWAYS, "Received a BAD packet !\n");
    return;
  }

  switch (nrclass->getVal()){

  case NRAttribute::INTEREST_CLASS:

    diffPrint(DEBUG_ALWAYS, "Received an Interest message !\n");
    break;

  case NRAttribute::DISINTEREST_CLASS:

    diffPrint(DEBUG_ALWAYS, "Received a Disinterest message !\n");
    break;

  default:

    diffPrint(DEBUG_ALWAYS, "Received an unknown message (%d)!\n",
	      nrclass->getVal());
    break;

  }
}

handle setupInterest(NR *dr)
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::NE, NRAttribute::DATA_CLASS));
  attrs.push_back(NRScopeAttr.make(NRAttribute::IS, NRAttribute::NODE_LOCAL_SCOPE));
  attrs.push_back(TargetAttr.make(NRAttribute::EQ, "F117A"));
  attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
  attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

handle setupPublication(NR *dr)
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::DATA_CLASS));
  attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
  attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));
  attrs.push_back(TargetAttr.make(NRAttribute::IS, "F117A"));

  handle h = dr->publish(&attrs);

  ClearAttrs(&attrs);

  return h;
}

void usage()
{
  diffPrint(DEBUG_ALWAYS, "Usage: agent [-p port] [-h]\n\n");
  diffPrint(DEBUG_ALWAYS, "\t-p - Uses port 'port' to talk to diffusion\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

int main(int argc, char **argv)
{
  NR *dr;
  int retval;
  int counter = 0;
  NRAttrVec data_attr;
  NRSimpleAttribute<int> *counterAttr;
  handle sendHandle;
  handle subscribeHandle;
  char input;
  int opt;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;
#ifdef INTERACTIVE
  fd_set FDS;
#endif // INTERACTIVE

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  // Parse command line options
  opterr = 0;

  while (1){
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS, "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage();

      break;

    }

    if (opt == -1)
      break;
  }

  dr = NR::createNR(diffusion_port);
  mr = new MyReceive;

  subscribeHandle = setupInterest(dr);
  sendHandle = setupPublication(dr);

  data_attr.push_back(AppStringAttr.make(NRAttribute::IS, "Sensing is working !\0"));
  data_attr.push_back(AppDummyAttr.make(NRAttribute::EQ, 2001));
  counterAttr = AppCounterAttr.make(NRAttribute::IS, counter);
  data_attr.push_back(counterAttr);

  // Main thread will send stuff
  while(1){
#ifdef INTERACTIVE
    FD_SET(0, &FDS);
    printf("Now what ? ");
    fflush(NULL);
    select(1, &FDS, NULL, NULL, NULL);
    input = getc(stdin);
    printf("\n");
    if (input == 'c'){
      dr->unsubscribe(subscribeHandle);
      exit(0);
    }
#else
    sleep(5);
#endif
    diffPrint(DEBUG_ALWAYS, "Sending Data %d\n", counter);
    retval = dr->send(sendHandle, &data_attr);

    counter++;

    counterAttr->setVal(counter);
  }

  return 0;
}
