// *********************************************************
//
// comApp1.cc     : Command Application multiple queries
// author         : Fabio Silva
//
// $Id: comApp1.cc,v 1.1 2001/11/21 19:51:25 fabio Exp $
//
// *********************************************************

#include "nestedBAE.hh"

extern char *application_id;

ComApp *com_app;

int MyTimerReceive::recv(handle hdl, void *p)
{
  return com_app->ProcessTimers(hdl, p);
}

void MyTimerReceive::del(void *p)
{
  // Nothing to do !
}

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  com_app->recv(data, my_handle);
}

int ComApp::ProcessTimers(handle hdl, void *p)
{
  TimerType *timer;
  int reqId;

  timer = (TimerType *) p;

  switch (timer->which_timer){

  case REQUEST_EXPIRE_TIMER:

    reqId = (int) timer->param;

    ProcessExpiredRequest(reqId);

    return -1;

    break;
  }
  return 0;
}

void ComApp::recv(NRAttrVec *data, handle h)
{
  if (h == subscribeSensorHandle){
    ProcessSensorInfo(data);
    return;
  }

  ProcessImagerData(data);
}

void ComApp::ProcessExpiredRequest(int reqId)
{
  PendingStateEntry *entry = NULL;
  struct timeval tv;
  int retval;

  getTime(&tv);

  entry = FindPendingSubscription(reqId);
  if (entry){
    diffPrint(DEBUG_ALWAYS, "Request #%d timed out !\n", reqId);

#ifdef DEBUG
    if (outfile){
      fprintf(outfile, "Node %d, request %d timed out at %.6lf !\n",
	      node_id, reqId, (double) tv.tv_sec + (0.000001 * tv.tv_usec));
      fflush(NULL);
    }
#endif    

    retval = dr->unsubscribe(entry->subscriptionHandle);

    if (retval != OK)
      diffPrint(DEBUG_ALWAYS, "Failed to unsubscribe !\n");

    delete entry;
  }
}

void ComApp::ProcessSensorInfo(NRAttrVec *data)
{
  int sensor_id;
  int sensor_seq;
  NRSimpleAttribute<int> *nodeIdAttr;
  NRSimpleAttribute<int> *counterAttr;
  NRSimpleAttribute<void *> *timeAttr;
  PendingStateEntry *pending_entry;
  EventTime *sensor_event_time;
  TimerType *timer;
  struct timeval tv;

  // Get node Id and light State
  nodeIdAttr = NodeIdAttr.find(data);
  counterAttr = AppCounterAttr.find(data);
  timeAttr = TimeAttr.find(data);

  if (!nodeIdAttr || !counterAttr || !timeAttr){
    diffPrint(DEBUG_ALWAYS,
	      "Message received from sensor is broken !\n");
    return;
  }

  getTime(&tv);

  sensor_event_time = (EventTime *) timeAttr->getVal();

  sensor_seq = counterAttr->getVal();
  sensor_id = nodeIdAttr->getVal();

  sensor_counter++;

  // Output message received from sensor
  diffPrint(DEBUG_ALWAYS, "Received CPA %d from Sensor %d !\n",
	    sensor_seq, sensor_id);

  interest_counter++;

  diffPrint(DEBUG_ALWAYS, "Sending Request Request #%d !\n",
	    interest_counter);

#ifdef DEBUG
  if (outfile){
    fprintf(outfile, "Node %d rcvd CPA %d from sensor %d at time: %.6lf !\n",
	    node_id, sensor_seq, sensor_id, (double) tv.tv_sec +
	    (0.000001 * tv.tv_usec));
    fprintf(outfile, "Node %d sending request %d !\n", node_id,
	    interest_counter);
    fflush(NULL);
      }
#endif

  // Create new request entry
  pending_entry = new PendingStateEntry;
  pending_entry->request_id = interest_counter;
  pending_entry->subscriptionHandle = setupImagerInterest();
  pending_entry->seconds = sensor_event_time->seconds;
  pending_entry->useconds = sensor_event_time->useconds;

  // Add request to the pending requests
  pendingRequests.push_back(pending_entry);

  timer = new TimerType(REQUEST_EXPIRE_TIMER);
  timer->param = (void *) interest_counter;
  ((DiffusionRouting *)dr)->addTimer(REQUEST_EXPIRATION_TIME,
				     (void *) timer, tcb);
}

void ComApp::ProcessImagerData(NRAttrVec *data)
{
  NRSimpleAttribute<int> *counterAttr = NULL;
  NRSimpleAttribute<int> *requestAttr = NULL;
  PendingStateEntry *entry = NULL;
  int sensor_seq;
  struct timeval tmv;
  int retval;
  int request_number;
  long delay_seconds;
  long delay_useconds;
  float total_delay;

  counterAttr = AppCounterAttr.find(data);
  requestAttr = UserRqIdAttr.find(data);

  if (!counterAttr || !requestAttr){
    diffPrint(DEBUG_ALWAYS, "Received a broken message !\n");
    printAttrs(data);
    return;
  }

  getTime(&tmv);

  // Read Attributes
  request_number = requestAttr->getVal();
  sensor_seq = counterAttr->getVal();

  entry = FindPendingSubscription(request_number);

  // If entry found, this is still a valid request
  if (entry){
    diffPrint(DEBUG_ALWAYS, "Received Back Imager Request #%d ! \n",
	      request_number);

    imager_counter++;

    // Calculate latency
    delay_seconds = tmv.tv_sec;
    delay_useconds = tmv.tv_usec;

    if ((delay_seconds < entry->seconds) ||
	((delay_seconds == entry->seconds) &&
	 (delay_useconds < entry->useconds))){
      // Something wrong with the time
      diffPrint(DEBUG_IMPORTANT, "Error calculating delay !\n");
      delay_useconds = 0;
      delay_seconds = -1;
    }
    else{
      delay_seconds = delay_seconds - entry->seconds;
      if (delay_useconds < entry->useconds){
	delay_seconds--;
	delay_useconds = delay_useconds + 1000000;
      }
      delay_useconds = delay_useconds - entry->useconds;
    }

    total_delay = (float) (1.0 * delay_seconds) + ((float) delay_useconds / 1000000.0);

    diffPrint(DEBUG_ALWAYS, "Delay = %f\n", total_delay);

#ifdef DEBUG
  if (outfile){
    fprintf(outfile, "Node %d received image request %d at time: %.6lf !\n",
	    node_id, request_number, (double) tmv.tv_sec +
	    (0.000001 * tmv.tv_usec));
    fprintf(outfile, "Calculated Delay = %f\n", total_delay);
    fflush(NULL);
      }
#endif

    retval = dr->unsubscribe(entry->subscriptionHandle);
      
    if (retval != OK)
      diffPrint(DEBUG_ALWAYS, "Failed to unsubscribe !\n");

    delete entry;
  }
  else{
    diffPrint(DEBUG_ALWAYS, "Received Expired Acoustic Sample %d !\n",
	      request_number);
  }
}

PendingStateEntry *ComApp::FindPendingSubscription(int reqId)
{
  PendingRequestsList::iterator itr;
  PendingStateEntry *entry;

  for (itr = pendingRequests.begin(); itr != pendingRequests.end(); ++itr){
    entry = *itr;
    if (entry->request_id == reqId){
      itr = pendingRequests.erase(itr);
      return entry;
    }
  }

  // Request not found !
  return NULL;
}

handle ComApp::setupImagerInterest()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::EQ, "img"));
  attrs.push_back(UserRqIdAttr.make(NRAttribute::IS, interest_counter));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

handle ComApp::setupSensorInterest()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::EQ, "pir"));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

ComApp::ComApp(int _node_id, u_int16_t port)
{
  dr = NR::createNR(port);
  mr = new MyReceive;
  tcb = new MyTimerReceive;
  node_id = _node_id;
  sensor_counter = 0;
  imager_counter = 0;
  interest_counter = 0;

  // Setup interest
  subscribeSensorHandle = setupSensorInterest();

#ifdef DEBUG
  char filename[100];

  snprintf(filename, 100, "/sensit/out/isi/isi-w-com1-%d", node_id);
  outfile = fopen(filename, "w");
#endif
  
  diffPrint(DEBUG_ALWAYS, "Command Application starting...\n");
}

void ComApp::run()
{
  while (1){
    // This thread does nothing !
    sleep (1000);
  }
}

void usage(char *s)
{
  diffPrint(DEBUG_ALWAYS, "Usage: %s [-p port] [-h]\n\n", s);
  diffPrint(DEBUG_ALWAYS, "\t-p - Specifies the diffusion port on the node\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

int main(int argc, char **argv)
{
  int id;
  int opt;
  char *scadds;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  scadds = getenv("scadds_addr");
  if (scadds == NULL){
    diffPrint(DEBUG_ALWAYS, "Error: Can't get node id !\n");
    exit(-1);
  }

  id = atoi(scadds);

  // Parse command line options
  opterr = 0;

  while (1) {
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS,
		  "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage(argv[0]);

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage(argv[0]);

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage(argv[0]);

      break;

   }

    if (opt == -1)
      break;
  }

  com_app = new ComApp(id, diffusion_port);
  com_app->run();

  return 0;
}
