// *********************************************************
//
// imgApp1.cc      : Imager App for multiple queries case
// author          : Fabio Silva
//
// $Id: imgApp1.cc,v 1.1 2001/11/21 19:51:25 fabio Exp $
//
// *********************************************************

#include "nestedBAE.hh"
#include <unistd.h>

extern char *application_id;

ImagerApp *imager_app;

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  imager_app->recv(data, my_handle);
}

void ImagerApp::recv(NRAttrVec *data, handle h)
{
  if (h == userSubscribeHandle){
    ProcessUserRequest(data);
    return;
  }

  diffPrint(DEBUG_ALWAYS, "Received a message for an unknown handle !\n");
}

void ImagerApp::ProcessUserRequest(NRAttrVec *data)
{
  NRAttrVec *attrs;
  NRSimpleAttribute<int> *nrclass = NULL;
  NRSimpleAttribute<int> *requestId = NULL;
  struct timeval tv;
  int retval;
  int request_number;

  nrclass = NRClassAttr.find(data);
  requestId = UserRqIdAttr.find(data);

  if (!nrclass || !requestId){
    diffPrint(DEBUG_ALWAYS, "Received a broken message !\n");
    return;
  }

  getTime(&tv);

  switch (nrclass->getVal()){

  case NRAttribute::INTEREST_CLASS:

    request_number = requestId->getVal();

    diffPrint(DEBUG_ALWAYS, "Received request %d from the user !\n",
	      request_number);

    attrs = new NRAttrVec;
    attrs->push_back(UserRqIdAttr.make(NRAttribute::EQ,
				       request_number));

    attrs->push_back(AppCounterAttr.make(NRAttribute::IS, send_counter));
    diffPrint(DEBUG_ALWAYS, "Sending image %d \n", send_counter);

#ifdef DEBUG
    if (outfile){
      fprintf(outfile, "Node %d, sending image %d in response to request %d, time: %.6lf !\n", node_id, send_counter, request_number, (double) tv.tv_sec +
	      (0.000001 * tv.tv_usec));
      fflush(NULL);
    }
#endif

    // Send the attributes to the sink
    retval = dr->send(publishHandle, attrs);
    send_counter++;

    ClearAttrs(attrs);
    delete attrs;

    break;

  default:

    diffPrint(DEBUG_ALWAYS,
	      "Received an unknown message (%d)!\n", nrclass->getVal());
    break;

  }
}

handle ImagerApp::setupUserInterest()
{
  NRAttrVec attrs;
 
  attrs.push_back(NRClassAttr.make(NRAttribute::EQ,
				   NRAttribute::INTEREST_CLASS));
  attrs.push_back(NRScopeAttr.make(NRAttribute::IS,
				   NRAttribute::NODE_LOCAL_SCOPE));
  attrs.push_back(TaskAttr.make(NRAttribute::IS, "img"));
 
  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

handle ImagerApp::setupPublication()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::DATA_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::IS, "img"));

  handle h = dr->publish(&attrs);

  ClearAttrs(&attrs);

  return h;
}

char * ImagerApp::getImage()
{
  return strdup("Look at this tank !");
}

ImagerApp::ImagerApp(int _node_id, u_int16_t port)
{
  dr = NR::createNR(port);
  mr = new MyReceive;

  // Initialize Acoustic Application
  send_counter = 0;
  node_id = _node_id;

  userSubscribeHandle = setupUserInterest();
  publishHandle = setupPublication();

#ifdef DEBUG
  char filename[100];

  snprintf(filename, 100, "/sensit/out/isi/isi-w-img1-%d", node_id);
  outfile = fopen(filename, "w");
#endif

  diffPrint(DEBUG_ALWAYS, "Imager App starting...\n");
}

void usage(char *s)
{
  diffPrint(DEBUG_ALWAYS, "Usage: %s [-p port]\n\n", s);
  diffPrint(DEBUG_ALWAYS, "\t-p - Specifies the diffusion port on the node\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

void ImagerApp::run()
{
  // Do nothing
  while(1)
    sleep(100);
}

int main(int argc, char **argv)
{
  int id;
  int opt;
  char *scadds;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  scadds = getenv("scadds_addr");
  if (scadds == NULL){
    diffPrint(DEBUG_ALWAYS, "Error: Can't get node id !\n");
    exit(-1);
  }

  id = atoi(scadds);

  // Parse command line options
  opterr = 0;

  while (1) {
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS, "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage(argv[0]);

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage(argv[0]);

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage(argv[0]);

      break;

   }

    if (opt == -1)
      break;
  }

  imager_app = new ImagerApp(id, diffusion_port);
  imager_app->run();

  return 0;
}
