// *********************************************************
//
// lightApp.cc    : Light App for nested queries
// author         : Fabio Silva
//
// $Id: lightApp.cc,v 1.1 2001/11/21 19:51:26 fabio Exp $
//
// *********************************************************

#include "nested.hh"
#include <unistd.h>

extern char *application_id;

LightApp *light_app;

int MyTimerReceive::recv(handle hdl, void *p)
{
  return light_app->ProcessTimers(hdl, p);
}

void MyTimerReceive::del(void *p)
{
  // Nothing to delete !
}

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  light_app->recv(data, my_handle);
}

void LightApp::recv(NRAttrVec *data, handle h)
{
  struct timeval tmv;
  NRSimpleAttribute<int> *nrclass = NULL;

  getTime(&tmv);

  if (h == subscribeHandle){
    nrclass = NRClassAttr.find(data);

    if (nrclass){
      switch (nrclass->getVal()){

      case NRAttribute::INTEREST_CLASS:

	diffPrint(DEBUG_ALWAYS, "Tasked by user !\n");
	fprintf(ofile, "%ld:%ld - Node %d tasked by sink\n", tmv.tv_sec, tmv.tv_usec, node_id);
	fflush(NULL);
	tasked++;

	break;

      case NRAttribute::DISINTEREST_CLASS:

	diffPrint(DEBUG_ALWAYS, "Task expired !\n");
	fprintf(ofile, "%ld:%ld - Node %d untasked by sink\n", tmv.tv_sec, tmv.tv_usec, node_id);
	fflush(NULL);
	tasked--;

	break;

      default:

	diffPrint(DEBUG_ALWAYS, "Received unknown message !\n");

	break;
      }
    }
    else{
      diffPrint(DEBUG_ALWAYS, "Received a broken message !\n");
    }
  }
  else{
    diffPrint(DEBUG_ALWAYS, "Received a message for an unknown handle !\n");
  }
}

int LightApp::ProcessTimers(handle hdl, void *p)
{
  struct timeval tmv;
  TimerType *timer;
  int retval;

  timer = (TimerType *) p;

  switch (timer->which_timer){

  case LIGHT_SEND_TIMER:

    if (tasked > 0){
      getTime(&tmv);
      currentState->light_state = light_state;
      counterAttr->setVal(counter);
      fprintf(ofile, "%ld:%ld - SND - N: %d - P: %d - L: ", tmv.tv_sec, tmv.tv_usec, node_id, counter);
      if (light_state == LIGHT_IS_ON){
	fprintf(ofile, "ON !\n");
      }
      else{
	fprintf(ofile, "OFF !\n");
      }
      fflush(NULL);

      // Send light state
      retval = dr->send(publishHandle, &data_attr);
      counter++;

      return 0;
    }

    diffPrint(DEBUG_ALWAYS, "Not tasked !\n");

    return 0;
  
    break;

  case LIGHT_CHANGE_TIMER:

    // Change light state
    if (light_state == LIGHT_IS_ON){
      light_state = LIGHT_IS_OFF;
      diffPrint(DEBUG_ALWAYS, "Light now is OFF !\n");
    }
    else{
      light_state = LIGHT_IS_ON;
      diffPrint(DEBUG_ALWAYS, "Light now is ON !\n");
    }

    getTime(&tmv);
    lastEventTime->seconds = tmv.tv_sec;
    lastEventTime->useconds = tmv.tv_usec;

    events_counter++;

    if (tasked > 0){
      getTime(&tmv);
      currentState->light_state = light_state;
      counterAttr->setVal(counter);
      fprintf(ofile, "%ld:%ld - SND - N: %d - P: %d - L: ", tmv.tv_sec, tmv.tv_usec, node_id, counter);
      if (light_state == LIGHT_IS_ON){
	fprintf(ofile, "ON !\n");
      }
      else{
	fprintf(ofile, "OFF !\n");
      }
      fflush(NULL);

      // Send light state
      retval = dr->send(publishHandle, &data_attr);
      counter++;
    }

    return 0;

    break;

  case STOP_TIMER:

    timetoStop();
    exit(0);

    break;
  }
}

void LightApp::timetoStop()
{
  struct timeval tv;

  getTime(&tv);
  fprintf(ofile, "%ld:%ld - Light Source %d stopping\n",
	  tv.tv_sec, tv.tv_usec, node_id);
  fprintf(ofile, "Total Events sent   : %d\n", events_counter);
  fprintf(ofile, "Total messages sent : %d\n", counter);
  fflush(NULL);
  fclose(ofile);
}

handle LightApp::setupPublication()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::DATA_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::IS, "li"));

  handle h = dr->publish(&attrs);

  ClearAttrs(&attrs);

  return h;
}

handle LightApp::setupSubscription()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::NE, NRAttribute::DATA_CLASS));
  attrs.push_back(NRScopeAttr.make(NRAttribute::IS, NRAttribute::NODE_LOCAL_SCOPE));
  attrs.push_back(TaskAttr.make(NRAttribute::IS, "li"));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

LightApp::LightApp(int _node_id, long _stop_time, u_int16_t port)
{
  struct timeval tmv;
  TimerType *timer;

  // Initialize
  node_id = _node_id;
  stop_time = _stop_time;
  tasked = 0;
  counter = 0;
  events_counter = 0;
  light_state = LIGHT_IS_OFF;
  getTime(&tmv);
  srand(tmv.tv_usec);

  // Create classes and set up publication
  dr = NR::createNR(port);
  mr = new MyReceive;
  tcb = new MyTimerReceive;
  publishHandle = setupPublication();
  subscribeHandle = setupSubscription();

  // Open output file
  ofile = fopen("/tmp/light-src.out","w");
  if (ofile == NULL){
    diffPrint(DEBUG_ALWAYS, "Can't open log file !\n");
    exit(-1);
  }

  getTime(&tmv);
  fprintf(ofile, "%ld:%ld - Light Source %d starting\n", tmv.tv_sec, tmv.tv_usec, node_id);
  fflush(NULL);

  lastEventTime = new EventTime;
  lastEventTime->seconds = tmv.tv_sec;
  lastEventTime->useconds = tmv.tv_usec;

  currentState = new Light_Node_State;
  currentState->node_id = node_id;
  currentState->light_state = light_state;

  // Create light state attribute
  lightAttr = LightStateAttr.make(NRAttribute::IS, (void *)currentState, sizeof(Light_Node_State));
  data_attr.push_back(lightAttr);

  // Create counter attribute
  counterAttr = AppCounterAttr.make(NRAttribute::IS, counter);
  data_attr.push_back(counterAttr);

  // Create time attribute
  timeAttr = TimeAttr.make(NRAttribute::IS, (void *) lastEventTime, sizeof(EventTime));
  data_attr.push_back(timeAttr);

  // Change pointers to point to attributes' copy
  delete currentState;
  currentState = (Light_Node_State *) lightAttr->getVal();

  delete lastEventTime;
  lastEventTime = (EventTime *) timeAttr->getVal();

  // Stop this application after 'stop_time' seconds
  if (stop_time > 0){
    timer = new TimerType(STOP_TIMER);
    ((DiffusionRouting *)dr)->addTimer(stop_time * 1000, (void *) timer, tcb);
  }

  timer = new TimerType(LIGHT_SEND_TIMER);
  ((DiffusionRouting *)dr)->addTimer(LIGHT_SENDING_PERIOD, (void *) timer, tcb);
  timer = new TimerType(LIGHT_CHANGE_TIMER);
  ((DiffusionRouting *)dr)->addTimer(LIGHT_CHANGING_PERIOD, (void *) timer, tcb);
}

void LightApp::run()
{
  // Main thread does nothing !
  while(1){
    sleep(100);
  }
}

void usage()
{
  diffPrint(DEBUG_ALWAYS, "Usage: lightapp [-t stoptime] [-p port] [-h]\n\n");
  diffPrint(DEBUG_ALWAYS, "\t-t - Schedule light app to exit after stoptime seconds\n");
  diffPrint(DEBUG_ALWAYS, "\t-p - Specifies the diffusion port on the node\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

int main(int argc, char **argv)
{
  int id;
  int opt;
  long stop_time = 0;
  char *scadds;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  scadds = getenv("scadds_addr");
  if (scadds == NULL){
    diffPrint(DEBUG_ALWAYS, "Error: Can't get node id !\n");
    exit(-1);
  }

  id = atoi(scadds);

  // Parse command line options
  opterr = 0;

  while (1) {
    opt = getopt(argc, argv, "ht:p:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS,
		  "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 't':

      stop_time = atol(optarg);
      if (stop_time <= 0){
	diffPrint(DEBUG_ALWAYS, "Error: stop time must be > 0\n");
	exit(0);
      }
      else{
	diffPrint(DEBUG_ALWAYS,
		  "Light source will stop after %ld seconds\n", stop_time); 
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage();

      break;

   }

    if (opt == -1)
      break;
  }

  light_app = new LightApp(id, stop_time, diffusion_port);
  light_app->run();

  return 0;
}
