// *********************************************************
//
// nested.hh      : Nested queries include file
// author         : Fabio Silva
//
// $Id: nested.hh,v 1.1 2001/11/21 19:51:26 fabio Exp $
//
// *********************************************************

#ifndef nested_hh
#define nested_hh

#include <stdlib.h>
#include <stdio.h>
#include "dr.hh"

extern NRSimpleAttributeFactory<char *> TaskAttr;
extern NRSimpleAttributeFactory<void *> LightStateAttr;
extern NRSimpleAttributeFactory<int> AppCounterAttr;
extern NRSimpleAttributeFactory<void *> TimeAttr;
extern NRSimpleAttributeFactory<int> UserRqIdAttr;
extern NRSimpleAttributeFactory<int> NodeIdAttr;

class MyReceive : public NR::Callback {
public:
  void recv(NRAttrVec *data, NR::handle my_handle);
};

class MyTimerReceive : public TimerCallbacks {
public:
  int recv(handle hdl, void *p);
  void del(void *p);
};

class TimerType {
public:
  int  which_timer;
  void *param;

  TimerType(int _which_timer) : which_timer(_which_timer)
  {
    param = NULL;
  };

  ~TimerType() {};
};

#define TASK_KEY          5000
#define LIGHT_STATE_KEY   5001
#define COUNTER_KEY       5002
#define TIME_KEY          5003
#define USER_RQ_ID        5004
#define NODE_ID_KEY       5005

#define LIGHT_SENDING_PERIOD     6000 // in msec
#define LIGHT_CHANGING_PERIOD   60000 // in msec
#define REQUEST_EXPIRATION_TIME 45000 // in msec

#define LIGHT_SEND_TIMER     100
#define LIGHT_CHANGE_TIMER   101
#define REQUEST_EXPIRE_TIMER 102

#define LIGHT_IS_ON    100
#define LIGHT_IS_OFF   200

class EventTime {
public:
  long seconds;
  long useconds;
};

class Light_Node_State {
public:
  int node_id;
  int light_state;
};

class PendingStateEntry {
public:
  int request_id;
  long seconds;
  long useconds;
  handle subscriptionHandle;
};

class Node_Entry {
public:
  int sensor_id;
  int sensor_state;

  Node_Entry(int _sensor_id, int _sensor_state) : sensor_id(_sensor_id),
						  sensor_state(_sensor_state)
  {}
};

typedef list<Node_Entry *> NodeList;
typedef list<PendingStateEntry *> PendingRequestsList;

class LightApp {
public:
  LightApp(int _node_id, long _stop_time, u_int16_t port);
  void run();
  int ProcessTimers(handle hdl, void *p);
  void recv(NRAttrVec *data, NR::handle my_handle);

private:
  NR *dr;
  MyTimerReceive *tcb;
  MyReceive *mr;
  int node_id;
  long stop_time;
  int light_state;
  NRAttrVec data_attr;
  NRSimpleAttribute<void *> *lightAttr;
  NRSimpleAttribute<int> *counterAttr;
  NRSimpleAttribute<void *> *timeAttr;
  Light_Node_State *currentState;
  EventTime *lastEventTime;
  int tasked;
  int counter;
  int events_counter;
  FILE *ofile;

  // Handles
  handle publishHandle;
  handle subscribeHandle;

  // Methods
  handle setupPublication();
  handle setupSubscription();
  void timetoStop();
};

class AcousticApp {
public:
  AcousticApp(int _node_id, long _stop_time, u_int16_t port);
  int ProcessTimers(handle hdl, void *p);
  void run();
  void recv(NRAttrVec *data, handle h);

  MyReceive *mr;

private:
  // Class variables
  NR *dr;
  MyTimerReceive *tcb;
  int node_id;
  int tasked;
  int receive_counter;
  int send_counter;
  long stop_time;
  NodeList lightNodeList;
  FILE *ofile;

  // Handles
  handle publishHandle;
  handle userSubscribeHandle;
  handle lightInfoHandle;

  // Methods
  handle setupUserInterest();
  handle setupLightInterest();
  handle setupPublication();

  void timetoStop();
  void ProcessUserRequest(NRAttrVec *data);
  void ProcessLightInfo(NRAttrVec *data);

  // Management
  Node_Entry *FindLightNodeInfo(int id);
  void DeleteNodeInfo();
  char * getSoundSample();
};

class UserApp {
public:
  UserApp(int _node_id, long _stop_time, u_int16_t port);
  int ProcessTimers(handle hdl, void *p);
  void run();
  void recv(NRAttrVec *data, handle h);

  MyReceive *mr;

private:
  // Class variables
  NR *dr;
  MyTimerReceive *tcb;
  float lat, lon;
  int node_id;
  int interest_counter;
  int light_counter;
  int acoustic_counter;
  long stop_time;
  FILE *ofile;
  NodeList lightNodeList;
  PendingRequestsList pendingRequests;

  // Handles
  handle subscribeAcousticHandle;
  handle subscribeLightHandle;

  // Methods
  handle setupAcousticInterest();
  handle setupLightInterest();

  void timetoStop();
  void ProcessAcousticData(NRAttrVec *data);
  void ProcessLightInfo(NRAttrVec *data);
  void ProcessExpiredRequest(int reqId);

  Node_Entry *FindLightNodeInfo(int id);
  PendingStateEntry *FindPendingSubscription(int reqId);
};

#endif
