// *********************************************************
//
// nested.hh      : Nested queries include file
// author         : Fabio Silva
//
// $Id: nestedBAE.hh,v 1.1 2001/11/21 19:51:26 fabio Exp $
//
// *********************************************************

#ifndef nestedBAE_hh
#define nestedBAE_hh

#include <stdlib.h>
#include <stdio.h>
#include "dr.hh"

#include "ProcStruct.h"
#include "RepUtil.h"
#include "RepTypeSafe.h"
#include "SubscriptionManager.h"

extern NRSimpleAttributeFactory<char *> TaskAttr;
extern NRSimpleAttributeFactory<void *> LightStateAttr;
extern NRSimpleAttributeFactory<int> AppCounterAttr;
extern NRSimpleAttributeFactory<void *> TimeAttr;
extern NRSimpleAttributeFactory<int> UserRqIdAttr;
extern NRSimpleAttributeFactory<int> NodeIdAttr;

class MyReceive : public NR::Callback {
public:
  void recv(NRAttrVec *data, NR::handle my_handle);
};

class MyTimerReceive : public TimerCallbacks {
public:
  int recv(handle hdl, void *p);
  void del(void *p);
};

class TimerType {
public:
  int  which_timer;
  void *param;

  TimerType(int _which_timer) : which_timer(_which_timer)
  {
    param = NULL;
  };

  ~TimerType() {};
};

#define TASK_KEY          5000
#define LIGHT_STATE_KEY   5001
#define COUNTER_KEY       5002
#define TIME_KEY          5003
#define USER_RQ_ID        5004
#define NODE_ID_KEY       5005

#define REQUEST_EXPIRATION_TIME 45000 // in msec
#define REQUEST_EXPIRE_TIMER 102

class EventTime {
public:
  long seconds;
  long useconds;
};

class PendingStateEntry {
public:
  int request_id;
  long seconds;
  long useconds;
  handle subscriptionHandle;
};

typedef list<PendingStateEntry *> PendingRequestsList;

class SensorApp {
public:
  SensorApp(int _node_id, u_int16_t port);
  void run();
  void recv(NRAttrVec *data, NR::handle my_handle);
  void sensorCallback(REP_EVENT ev);

private:
#ifdef DEBUG
  FILE *outfile;
#endif
  NR *dr;
  MyReceive *mr;
  int node_id;
  int tasked;
  int events_counter;

  // Handles
  handle publishHandle;
  handle subscribeHandle;

  // Methods
  handle setupPublication();
  handle setupSubscription();
};

class ImagerApp {
public:
  ImagerApp(int _node_id, u_int16_t port);
  void run();
  void recv(NRAttrVec *data, handle h);

  MyReceive *mr;

private:
  // Class variables
#ifdef DEBUG
  FILE *outfile;
#endif
  NR *dr;
  int node_id;
  int tasked;
  int receive_counter;
  int send_counter;

  // Handles
  handle publishHandle;
  handle userSubscribeHandle;
  handle lightInfoHandle;

  // Methods
  handle setupUserInterest();
  handle setupLightInterest();
  handle setupPublication();

  void ProcessUserRequest(NRAttrVec *data);
  void ProcessSensorInfo(NRAttrVec *data);

  // Management
  char * getImage();
};

class ComApp {
public:
  ComApp(int _node_id, u_int16_t port);
  int ProcessTimers(handle hdl, void *p);
  void run();
  void recv(NRAttrVec *data, handle h);

  MyReceive *mr;

private:
  // Class variables
#ifdef DEBUG
  FILE *outfile;
#endif
  NR *dr;
  MyTimerReceive *tcb;
  float lat, lon;
  int node_id;
  int interest_counter;
  int sensor_counter;
  int imager_counter;
  PendingRequestsList pendingRequests;

  // Handles
  handle subscribeImagerHandle;
  handle subscribeSensorHandle;

  // Methods
  handle setupImagerInterest();
  handle setupSensorInterest();

  void ProcessImagerData(NRAttrVec *data);
  void ProcessSensorInfo(NRAttrVec *data);
  void ProcessExpiredRequest(int reqId);

  PendingStateEntry *FindPendingSubscription(int reqId);
};

#endif
