//
// agent1.cc      : Test agent #1 Main File
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: testApp.cc,v 1.1 2001/11/21 19:51:27 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "testApp.hh"

NR *diffrt = NULL;

ARLLocalReceive *ARLLocalRecv;
ARLTrackReceive *ARLTrackRecv;

NR::handle localPub;
NR::handle localSub;
NR::handle trackSub;

NRSimpleAttributeFactory<void *>ARLBlobAttr(NRAttribute::DATABLOCK_KEY, 
                                            NRAttribute::BLOB_TYPE);

NRSimpleAttributeFactory<char *>ARLSubNameAttr(SUB_NAME_KEY,
					       NRAttribute::STRING_TYPE);

NRSimpleAttributeFactory<char *>ARLQueueNameAttr(QUEUE_NAME_KEY, 
						 NRAttribute::STRING_TYPE);

NRSimpleAttributeFactory<char *> TargetAttr(NRAttribute::TARGET_KEY, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<int> AppDummyAttr(APP_KEY1, NRAttribute::INT32_TYPE);
NRSimpleAttributeFactory<int> AppCounterAttr(APP_KEY3, NRAttribute::INT32_TYPE);

void ARLLocalReceive::recv(NRAttrVec *data, NR::handle h){
  printf("received message, handle = %d\n", h);
  fflush(NULL);
}

void ARLTrackReceive::recv(NRAttrVec *data, NR::handle h){
  printf("received message, handle = %d\n", h);
  fflush(NULL);
}

bool SetLocalPubSub(){
  fprintf(stderr, 
          "ColTraNe: Setting Local Publications and Subscriptions.... \n");
  fflush(NULL);
  NRAttrVec localpubattrs;
  NRAttrVec localsubattrs;

  localpubattrs.push_back(NRClassAttr.make(NRAttribute::IS, 
                                           NRAttribute::DATA_CLASS));
  localpubattrs.push_back(LatitudeAttr.make(NRAttribute::IS,10.0));
  localpubattrs.push_back(LongitudeAttr.make(NRAttribute::IS,10.0));
  localpubattrs.push_back(ARLSubNameAttr.make(NRAttribute::IS,"Local"));
  
  localPub = diffrt->publish(&localpubattrs);
  ClearAttrs(&localpubattrs);
  if(localPub == -1)  
  {
    fprintf(stderr, "ColTraNe: Local Publication failed.... \n");
    fflush(NULL);
    return false;
  } 
  localsubattrs.push_back(NRClassAttr.make(NRAttribute::IS, 
                                           NRAttribute::INTEREST_CLASS));
  localsubattrs.push_back(LatitudeAttr.make(NRAttribute::GE,1.0));
  localsubattrs.push_back(LongitudeAttr.make(NRAttribute::GE,1.0));
  localsubattrs.push_back(LatitudeAttr.make(NRAttribute::LE,100.0));
  localsubattrs.push_back(LongitudeAttr.make(NRAttribute::LE,100.0));
  localsubattrs.push_back(ARLSubNameAttr.make(NRAttribute::EQ,"Local"));
  
  localSub = diffrt->subscribe(&localsubattrs,ARLLocalRecv);
  ClearAttrs(&localsubattrs);
  if(localSub == -1) 
  {
    fprintf(stderr, "ColTraNe: Local Subscription failed.... \n");
    fflush(NULL);
    return false;
  }
  fprintf(stderr, "ColTraNe: Local Subscription Handle = %d\n",localSub);
  fflush(NULL);
  return true;
}

bool SetTrackSub(){
  fprintf(stderr, "ColTraNe: Setting Tracking Subscription.... \n");
  fflush(NULL);
  NRAttrVec tracksubattrs;
  bool set = true;

  tracksubattrs.push_back(NRClassAttr.make(NRAttribute::IS, 
                                           NRAttribute::INTEREST_CLASS));
  tracksubattrs.push_back(LatitudeAttr.make(NRAttribute::IS,10.0));
  tracksubattrs.push_back(LongitudeAttr.make(NRAttribute::IS,10.0));
  tracksubattrs.push_back(ARLSubNameAttr.make(NRAttribute::IS,"Track"));
  tracksubattrs.push_back(ARLQueueNameAttr.make(NRAttribute::EQ_ANY,""));

  trackSub = diffrt->subscribe(&tracksubattrs,ARLLocalRecv);
  if(trackSub == -1) 
  {
    fprintf(stderr, "ColTraNe: Tracking Subscription failed.... \n");
    fflush(NULL);
    set = false;
  }
  fprintf(stderr, "ColTraNe: Tracking Subscription Handle = %d\n",trackSub);
  fflush(NULL);
  
  ClearAttrs(&tracksubattrs);
  return set;
}

void SendLocalData(){
  fprintf(stderr, "ColTraNe: Sending Local Data.... \n");
  fflush(NULL);
  NRAttrVec sendAttrs;

  struct timeval tv;

  bool sent = true;

  sendAttrs.push_back(ARLBlobAttr.make(NRAttribute::IS,
                                       &tv,
				       sizeof(struct timeval)));

  if (diffrt->send(localPub,&sendAttrs) == -1){
    fprintf(stderr, "ColTraNe: Sending Local Data Failed.... \n");
    fflush(NULL);
    sent = false;
  } else {
    fprintf(stderr, "ColTraNe: Sending Local Data Succeeded.... \n");
    fflush(NULL);
  }

  ClearAttrs(&sendAttrs);
}


int main(int argc, char **argv)
{
  int opt;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  diffrt = NR::createNR(diffusion_port);

  ARLLocalRecv = new ARLLocalReceive();
  ARLTrackRecv = new ARLTrackReceive();

  SetLocalPubSub();
  SetTrackSub();

  while (1){
    sleep(5);

    SendLocalData();
  }

  return 0;
}
