// *********************************************************
//
// userApp1.cc    : User Application multiple queries case
// author         : Fabio Silva
//
// $Id: userApp1.cc,v 1.1 2001/11/21 19:51:27 fabio Exp $
//
// *********************************************************

#include "nested.hh"

extern char *application_id;

UserApp *user_app;

int MyTimerReceive::recv(handle hdl, void *p)
{
  return user_app->ProcessTimers(hdl, p);
}

void MyTimerReceive::del(void *p)
{
  // Nothing to do !
}

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  user_app->recv(data, my_handle);
}

int UserApp::ProcessTimers(handle hdl, void *p)
{
  TimerType *timer;
  int reqId;

  timer = (TimerType *) p;

  switch (timer->which_timer){

  case REQUEST_EXPIRE_TIMER:

    reqId = (int) timer->param;

    ProcessExpiredRequest(reqId);

    return -1;

    break;

  case STOP_TIMER:

    timetoStop();
    exit(0);

    break;
  }
  return 0;
}

void UserApp::timetoStop()
{
  struct timeval tv;

  getTime(&tv);
  fprintf(ofile, "%ld:%ld - Sink %d stopping\n", tv.tv_sec, tv.tv_usec, node_id);
  fprintf(ofile, "Total light messages received : %d\n", light_counter);
  fprintf(ofile, "Total acoustic messages received : %d\n", acoustic_counter);
  fflush(NULL);
  fclose(ofile);
}

void UserApp::recv(NRAttrVec *data, handle h)
{
  if (h == subscribeLightHandle){
    ProcessLightInfo(data);
    return;
  }

  ProcessAcousticData(data);
}

void UserApp::ProcessExpiredRequest(int reqId)
{
  PendingStateEntry *entry = NULL;
  int retval;

  entry = FindPendingSubscription(reqId);
  if (entry){
    diffPrint(DEBUG_ALWAYS, "Request #%d timed out !\n", reqId);

    diffPrint(DEBUG_ALWAYS, "Unsubscribing handle = %d\n",
	      entry->subscriptionHandle);

    retval = dr->unsubscribe(entry->subscriptionHandle);

    if (retval != OK)
      diffPrint(DEBUG_ALWAYS, "Failed to unsubscribe !\n");

    delete entry;
  }
}

void UserApp::ProcessLightInfo(NRAttrVec *data)
{
  int sensor_id;
  int sensor_state;
  int sensor_seq;
  NRSimpleAttribute<void *> *lightAttr;
  NRSimpleAttribute<int> *counterAttr;
  NRSimpleAttribute<void *> *timeAttr;
  PendingStateEntry *pending_entry;
  Light_Node_State *light_state;
  EventTime *sensor_event_time;
  Node_Entry *entry;
  TimerType *timer;
  struct timeval tmv;

  // Get node Id and light State
  lightAttr = LightStateAttr.find(data);
  counterAttr = AppCounterAttr.find(data);
  timeAttr = TimeAttr.find(data);

  if (!lightAttr || !counterAttr || !timeAttr){
    diffPrint(DEBUG_ALWAYS,
	      "Message received from light sensor is broken !\n");
    return;
  }

  sensor_event_time = (EventTime *) timeAttr->getVal();
  light_state = (Light_Node_State *) lightAttr->getVal();
  sensor_seq = counterAttr->getVal();

  sensor_id = light_state->node_id;
  sensor_state = light_state->light_state;

  getTime(&tmv);

  light_counter++;

  // Output message received from light sensor
  diffPrint(DEBUG_ALWAYS, "%ld:%ld - RCVD - N: %d - P: %d - L: ",
	    tmv.tv_sec, tmv.tv_usec, sensor_id, sensor_seq);
  if (sensor_state == LIGHT_IS_ON){
    diffPrint(DEBUG_ALWAYS, "ON !\n");
  }
  else{
    diffPrint(DEBUG_ALWAYS, "OFF !\n");
  }

  entry = FindLightNodeInfo(sensor_id);
  
  // Add new entry if sensor id unknown
  if (!entry){
    diffPrint(DEBUG_ALWAYS, "Adding node %d to the list !\n", sensor_id);
    entry = new Node_Entry(sensor_id, sensor_state);
    lightNodeList.push_back(entry);
    return;
  }

  if (entry->sensor_state != sensor_state){

    // Update sensor state
    entry->sensor_state = sensor_state;

    interest_counter++;

    diffPrint(DEBUG_ALWAYS, "%ld:%ld - Sending Request #%d\n",
	      tmv.tv_sec, tmv.tv_usec, interest_counter);

    // Create new request entry
    pending_entry = new PendingStateEntry;
    pending_entry->request_id = interest_counter;
    pending_entry->subscriptionHandle = setupAcousticInterest();
    pending_entry->seconds = sensor_event_time->seconds;
    pending_entry->useconds = sensor_event_time->useconds;

    diffPrint(DEBUG_ALWAYS, "Subscribed for Acoustic data, handle = %d !\n",
	      pending_entry->subscriptionHandle);

    // Add request to the pending requests
    pendingRequests.push_back(pending_entry);

    timer = new TimerType(REQUEST_EXPIRE_TIMER);
    timer->param = (void *) interest_counter;
    ((DiffusionRouting *)dr)->addTimer(REQUEST_EXPIRATION_TIME,
				       (void *) timer, tcb);
  }
}

void UserApp::ProcessAcousticData(NRAttrVec *data)
{
  NRSimpleAttribute<int> *counterAttr = NULL;
  NRSimpleAttribute<int> *requestAttr = NULL;
  PendingStateEntry *entry = NULL;
  int sensor_seq;
  struct timeval tmv;
  int retval;
  int request_number;
  long delay_seconds;
  long delay_useconds;
  float total_delay;

  counterAttr = AppCounterAttr.find(data);
  requestAttr = UserRqIdAttr.find(data);

  if (!counterAttr || !requestAttr){
    diffPrint(DEBUG_ALWAYS, "Received a broken message !\n");
    printAttrs(data);
    return;
  }

  getTime(&tmv);

  // Read Attributes
  request_number = requestAttr->getVal();
  sensor_seq = counterAttr->getVal();

  entry = FindPendingSubscription(request_number);

  // If entry found, this is still a valid request
  if (entry){
    diffPrint(DEBUG_ALWAYS, "%ld:%ld - RCVD - Acoustic - P: %d !\n",
	      tmv.tv_sec, tmv.tv_usec, sensor_seq);
    diffPrint(DEBUG_ALWAYS, "%ld:%ld - Received Request #%d\n",
	      tmv.tv_sec, tmv.tv_usec, request_number);

    acoustic_counter++;

    // Calculate latency
    delay_seconds = tmv.tv_sec;
    delay_useconds = tmv.tv_usec;

    if ((delay_seconds < entry->seconds) ||
	((delay_seconds == entry->seconds) &&
	 (delay_useconds < entry->useconds))){
      // Something wrong with the time
      diffPrint(DEBUG_ALWAYS, "Error calculating delay !\n");
    }
    else{
      delay_seconds = delay_seconds - entry->seconds;
      if (delay_useconds < entry->useconds){
	delay_seconds--;
	delay_useconds = delay_useconds + 1000000;
      }
      delay_useconds = delay_useconds - entry->useconds;
    }

    total_delay = (float) (1.0 * delay_seconds) + ((float) delay_useconds / 1000000.0);

    diffPrint(DEBUG_ALWAYS, "Delay = %f\n", total_delay);

    diffPrint(DEBUG_ALWAYS, "Unsubscribing handle = %d\n",
	      entry->subscriptionHandle);

    retval = dr->unsubscribe(entry->subscriptionHandle);
      
    if (retval != OK)
      diffPrint(DEBUG_ALWAYS, "Failed to unsubscribe !\n");

    delete entry;
  }
  else{
    diffPrint(DEBUG_ALWAYS, "%ld:%ld - Received Expired Acoustic Sample !\n",
	      tmv.tv_sec, tmv.tv_usec);
  }
}

Node_Entry *UserApp::FindLightNodeInfo(int id)
{
  NodeList::iterator itr;
  Node_Entry *entry = NULL;

  for (itr = lightNodeList.begin(); itr != lightNodeList.end(); ++itr){
    entry = *itr;
    if (entry->sensor_id == id)
      return entry;
  }

  // Light sensor state not found !
  return NULL;
}

PendingStateEntry *UserApp::FindPendingSubscription(int reqId)
{
  PendingRequestsList::iterator itr;
  PendingStateEntry *entry;

  for (itr = pendingRequests.begin(); itr != pendingRequests.end(); ++itr){
    entry = *itr;
    if (entry->request_id == reqId){
      itr = pendingRequests.erase(itr);
      return entry;
    }
  }

  // Request not found !
  return NULL;
}

handle UserApp::setupAcousticInterest()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::EQ, "ac"));
  attrs.push_back(UserRqIdAttr.make(NRAttribute::IS, interest_counter));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

handle UserApp::setupLightInterest()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::EQ, "li"));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

UserApp::UserApp(int _node_id, long _stop_time, u_int16_t port)
{
  struct timeval tmv;
  TimerType *timer;

  dr = NR::createNR(port);
  mr = new MyReceive;
  tcb = new MyTimerReceive;
  node_id = _node_id;
  stop_time = _stop_time;
  light_counter = 0;
  acoustic_counter = 0;
  interest_counter = 0;

  // Setup interest
  subscribeLightHandle = setupLightInterest();
  
  // Open output file
  ofile = fopen("/tmp/user-sink.out","w");
  if (ofile == NULL){
    diffPrint(DEBUG_ALWAYS, "Can't open log file !\n");
    exit(-1);
  }

  getTime(&tmv);
  diffPrint(DEBUG_ALWAYS, "%ld:%ld - User Application %d starting\n",
	    tmv.tv_sec, tmv.tv_usec, node_id);

  // Stop this application after 'stop_time' seconds
  if (stop_time > 0){
    timer = new TimerType(STOP_TIMER);
    ((DiffusionRouting *)dr)->addTimer(stop_time * 1000, (void *) timer, tcb);
  }
}

void UserApp::run()
{
  while (1){
    // This thread does nothing !
    sleep (1000);
  }
}

void usage()
{
  diffPrint(DEBUG_ALWAYS, "Usage: userapp [-t stoptime] [-h]\n\n");
  diffPrint(DEBUG_ALWAYS, "\t-t - Schedule user app to exit after stoptime seconds\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

int main(int argc, char **argv)
{
  int id;
  int opt;
  long stop_time = 0;
  char *scadds;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  scadds = getenv("scadds_addr");
  if (scadds == NULL){
    diffPrint(DEBUG_ALWAYS, "Error: Can't get node id !\n");
    exit(-1);
  }

  id = atoi(scadds);

  // Parse command line options
  opterr = 0;

  while (1) {
    opt = getopt(argc, argv, "ht:p:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS,
		  "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 't':

      stop_time = atol(optarg);
      if (stop_time <= 0){
	diffPrint(DEBUG_ALWAYS, "Source Error: stop time must be > 0\n");
	exit(0);
      }
      else{
	diffPrint(DEBUG_ALWAYS,
		  "User app will stop after %ld seconds\n", stop_time); 
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage();

      break;

   }

    if (opt == -1)
      break;
  }

  user_app = new UserApp(id, stop_time, diffusion_port);
  user_app->run();

  return 0;
}
