

#include <unistd.h>
#include <stdio.h>
#include <sys/time.h>


#include "libfdb.h"
#include "detput.h"


#define MTEXT 5000


static void DetectionPutResult(void *data, void *arg)
{
  int dbid, status;

  dbid = CacheResultCode(data);
  status = CacheResultStatus(data);
  fprintf(stderr, "put result dbid=%d status=%d", dbid, status);
}


int DetectionPut(char *id, int channel, double time, 
		 int sensor, int direction, double confidence)
{
  char statement[MTEXT];
  int dbid;
  /*
   * Format the statement.
   */
  snprintf(statement, MTEXT,"put into detection (id, channel, time, \
        sensor, direction, confidence) values ('%s',%d,%.6lf,%d,%d,%g);",
	   id, channel, time, sensor, direction, confidence);
  /*
   * and do it.
   */
  dbid = CachePerform(statement, DetectionPutResult, 0);
  fprintf(stderr, "%d %s", dbid, statement);

  return dbid;
}


static void DetectionCreateResult(void *data, void *arg)
{
  int dbid, status;

  dbid = CacheResultCode(data);
  status = CacheResultStatus(data);
  fprintf(stderr, "create result dbid=%d status=%d", dbid, status);
}


int DetectionCreate()
{
  char statement[MTEXT];
  int dbid;
  /*
   * Each record is a single detection.
   */
  sprintf(statement,"create table detection (id c, channel i32, time f64, \
        sensor i32, direction i32, confidence f32, \
        primary(id, channel, time), expire 3600);");
  dbid=CachePerform(statement, DetectionCreateResult, 0);
  fprintf(stderr, "%d %s", dbid, statement);

  return dbid;
}


static void connected(int socket)
{
  fprintf(stderr, "connected to cache on socket %d", socket);
}


int DetectionInit()
{
  CacheConnect(connected);
  DetectionCreate();
}


