// *********************************************************
//
// gatewayApp.cc  : Gateway Application for Steel Knight
// author         : Fabio Silva
//
// $Id: gatewayApp.cc,v 1.1 2001/11/21 19:51:29 fabio Exp $
//
// *********************************************************

#include "sk.hh"
#include "detput.h"

extern char *application_id;

GatewayApp *gateway_app;

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  gateway_app->recv(data, my_handle);
}

void GatewayApp::recv(NRAttrVec *data, handle h)
{
  if (h == subscribeSensorHandle){
    ProcessSensorInfo(data);
    return;
  }
}

void GatewayApp::ProcessSensorInfo(NRAttrVec *data)
{
  NRSimpleAttribute<void *> *cpaAttr;
  CpaEvent *cpa_event;
  char buffer[100];
  int retval;

  // Get node Id and light State
  cpaAttr = CpaAttr.find(data);

  if (!cpaAttr){
    diffPrint(DEBUG_ALWAYS,
	      "Message received from sensor is broken !\n");
    return;
  }

  if (sensor_counter == 0){
    DetectionInit();
  
    for (int i = 0; i < 5; i++){
      CacheLoopTimeout(100);
    }
  }

  cpa_event = (CpaEvent *) cpaAttr->getVal();
  sensor_counter++;

  // Output message received from sensor
  diffPrint(DEBUG_ALWAYS, "Received CPA from node %d !\n",
	    cpa_event->node_id);

  snprintf(buffer, 100, "sensit%d", cpa_event->node_id);

  retval = DetectionPut(buffer, cpa_event->channel_id, cpa_event->cpa_time,
  			cpa_event->sensor_id, cpa_event->direction,
  			cpa_event->confidence);

  for (int i = 0; i < 5; i++){
    CacheLoopTimeout(100);
  }

#ifdef DEBUG
  if (outfile){
    fprintf(outfile, "Received event %d, node: %d, time: %.6lf !\n",
	    sensor_counter, cpa_event->node_id, cpa_event->cpa_time);
    fflush(NULL);
  }
#endif
}

handle GatewayApp::setupSensorInterest()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(TaskAttr.make(NRAttribute::EQ, "cpa"));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

GatewayApp::GatewayApp(int _node_id, u_int16_t port)
{
  dr = NR::createNR(port);
  mr = new MyReceive;
  node_id = _node_id;
  sensor_counter = 0;

  // Setup interest
  subscribeSensorHandle = setupSensorInterest();

#ifdef DEBUG
  char filename[100];

  snprintf(filename, 100, "/sensit/out/isi/isi-w-gw-%d", node_id);
  outfile = fopen(filename, "w");
#endif
  
  diffPrint(DEBUG_ALWAYS, "Gateway Application starting...\n");
}

void GatewayApp::run()
{
  while (1){
    // This thread does nothing !
    sleep (1000);
  }
}

void usage(char *s)
{
  diffPrint(DEBUG_ALWAYS, "Usage: %s [-p port] [-h]\n\n", s);
  diffPrint(DEBUG_ALWAYS, "\t-p - Specifies the diffusion port on the node\n");
  diffPrint(DEBUG_ALWAYS, "\t-h - Prints this information\n");
  diffPrint(DEBUG_ALWAYS, "\n");
  exit(0);
}

int main(int argc, char **argv)
{
  int id;
  int opt;
  char *scadds;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  application_id = strdup(argv[0]);

#ifdef BBN_LOGGER
  InitMainLogger();
#endif

  scadds = getenv("scadds_addr");
  if (scadds == NULL){
    diffPrint(DEBUG_ALWAYS, "Error: Can't get node id !\n");
    exit(-1);
  }

  id = atoi(scadds);

  // Parse command line options
  opterr = 0;

  while (1) {
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	diffPrint(DEBUG_ALWAYS,
		  "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage(argv[0]);

      break;

    case '?':

      diffPrint(DEBUG_ALWAYS, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage(argv[0]);

      break;

    case ':':

      diffPrint(DEBUG_ALWAYS, "Parameter missing !\n");
      usage(argv[0]);

      break;

   }

    if (opt == -1)
      break;
  }

  gateway_app = new GatewayApp(id, diffusion_port);
  gateway_app->run();

  return 0;
}
