/* VERSION WINSNG2-1_0 	 Jun 29 2001  */
/*
	rf.h- Header file for WINS NG v 2.0 RF API
    Copyright (C) 2000 Sensoria Corporation    
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 	Revision history:
    
*/   
#ifndef RF_H
#define RF_H
 
#include <stdio.h>
#include <sys/ioctl.h>

/*Allow linkage to C or C++*/
#ifdef __cplusplus
extern "C" {
#endif
 
typedef unsigned char  uint8;
typedef unsigned short uint16;

typedef struct RfHeader_t
{
  uint16 length;          /*Length in bytes of packet, not including header*/
  uint8  source[3];		 /*Address of original transmitting radio*/
  uint8  destination[3];  /*Address of final destination radio*/
  uint8  type;            /*Allows for type of packet*/
} RfHeader;

#define MAXRFDATASIZE 235
#define MAXRFPACKETSIZE 235 + sizeof(RfHeader)
#define MaxListBytes 60

typedef struct RfPacket_t
{
  RfHeader *header; 
  uint8    *data;
  uint8    *raw;
  size_t   bytesAlloc;
} RfPacket;

/*Allocate initial storage for a packet*/
int initializeRfPacket(RfPacket *p,size_t size);
/*Create RF Packet*/
int createRfPacket(RfPacket *p,RfHeader *header,uint8 *data, size_t size);

/*Define useful data types and functions for addresses*/
typedef uint8* Address;
void copyAddress(Address dest, Address src);
int compareAddress(Address a1, Address a2);
int readAddress(FILE* fp, Address address);
int sreadAddress(char* s, Address address);
void printAddress(FILE* fp,Address address);
void sprintAddress(char *c,Address address);

/*Define offsets into the packet*/

/*Define packet types (should be enum)*/
#define CONNECT        0
#define DISCONNECT     1
#define PING           2
#define ACK            3
#define STX            4
#define ETX            5
#define NODEID         6
#define NEIGHBORLIST   7
#define STARTSTREAM    8
#define STREAM         9
#define RESERVED_PACKET_TYPES 16

/*General usage functions*/
int readPacket(int fd,RfPacket *packet); /*Read a packet from driver*/
int sendPacket(int fd,RfPacket *packet); /*Send a packet to driver*/
int determineBaseRemote(int fd);
int getAddress(int fd, Address a);
int getConnectedList(int fd,uint8* list,int max);
int getRSSI(int fd);
int setBase(int fd);
int setRemote(int fd);
int getNetwork(int fd);
int setNetwork(int fd, int network);
int setTransmitPower(int fd, int Power);
int getTransmitPower(int fd);
int setNotifications(int fd, int setting);
int nextPacketLength(int fd);
int sleepRfModem(int fd);
int wakeRfModem(int fd);

/*Device Control interface*/
#define RFModem_Function_Class  0x18   

#define Set_Base_CtrlN                   1 
#define Set_Base_Ctrl                    _IO(RFModem_Function_Class, Set_Base_CtrlN)  
#define Set_Remote_CtrlN                 2
#define Set_Remote_Ctrl                  _IO(RFModem_Function_Class, Set_Remote_CtrlN)
#define Set_Network_CtrlN                3
#define Set_Network_Ctrl                _IOW(RFModem_Function_Class, Set_Network_CtrlN, 4)
#define Set_Transmit_Power_CtrlN         4
#define Set_Transmit_Power_Ctrl         _IOW(RFModem_Function_Class, Set_Transmit_Power_CtrlN, 4)
#define Determine_Network_CtrlN          5
#define Determine_Network_Ctrl          _IOR(RFModem_Function_Class, Determine_Network_CtrlN, 4)	 
#define Determine_Address_CtrlN          6
#define Determine_Address_Ctrl          _IOR(RFModem_Function_Class, Determine_Address_CtrlN, 3)
#define Determine_Base_Remote_CtrlN      7
#define Determine_Base_Remote_Ctrl      _IOR(RFModem_Function_Class, Determine_Base_Remote_CtrlN ,4)	 
#define Determine_Signal_Strength_CtrlN  8
#define Determine_Signal_Strength_Ctrl  _IOR(RFModem_Function_Class, Determine_Signal_Strength_CtrlN ,4)	 
#define Determine_Transmit_Power_CtrlN   9
#define Determine_Transmit_Power_Ctrl   _IOR(RFModem_Function_Class, Determine_Transmit_Power_CtrlN, 4)
#define Get_Connected_ListN              10
#define Get_Connected_List              _IOC(_IOC_READ,RFModem_Function_Class, Get_Connected_ListN,MaxListBytes) 
#define Set_NotificationsN               11
#define Set_Notifications               _IOW(RFModem_Function_Class, Set_NotificationsN , 1)
#define RF_Sleep_CtrlN                  12
#define RF_Sleep_Ctrl                   _IO(RFModem_Function_Class, RF_Sleep_CtrlN )
#define RF_Wake_CtrlN                   13
#define RF_Wake_Ctrl                    _IO(RFModem_Function_Class, RF_Wake_CtrlN )

#define Connect_Function_Class 0x10
#define Ctrl_PEEKLENGTH       0x0d
#define Ctrl_QueryFrontSize _IOR(Connect_Function_Class ,Ctrl_PEEKLENGTH,0)

/*Allow linkage to C or C++*/
#ifdef __cplusplus
}
#endif



#endif /*defined RFMODEM_PACKET_H */

